@@ -1,6 +1,7 @@
 
 from pyroute2.iproute import IPRoute
 from pyroute2.ipdb import IPDB
+from pyroute2.netns import NetNS
 from pyroute2.netlink.rtnl import IPRSocket
 from pyroute2.netlink.taskstats import TaskStats
 from pyroute2.netlink.ipq import IPQSocket
@@ -10,6 +11,7 @@ from pyroute2.netlink import NetlinkError
 modules = [IPRSocket,
            IPRoute,
            IPDB,
+           NetNS,
            TaskStats,
            IPQSocket,
            GenericNetlinkSocket,
