@@ -39,6 +39,18 @@ the programmer's perspective. Please read also
 `NetlinkSocket` documentation to know more about async
 mode.
 
+think about IPDB
+----------------
+
+If you plan to regularly fetch loads of objects, think
+about IPDB also. Unlike to IPRoute, IPDB does not fetch
+all the objects from OS every time you request them, but
+keeps a cache that is asynchronously updated by the netlink
+broadcasts. For a long-term running programs, that often
+retrieve info about hundreds or thousands of objects, it
+can be better to use IPDB as it will load CPU significantly
+less.
+
 classes
 -------
 '''
@@ -83,6 +95,8 @@ from pyroute2.netlink.rtnl import RTM_SETBRIDGE
 from pyroute2.netlink.rtnl import RTM_GETBRIDGE
 from pyroute2.netlink.rtnl import RTM_SETBOND
 from pyroute2.netlink.rtnl import RTM_GETBOND
+from pyroute2.netlink.rtnl import RTM_GETDHCP
+from pyroute2.netlink.rtnl import RTM_NEWTUNTAP
 from pyroute2.netlink.rtnl import TC_H_INGRESS
 from pyroute2.netlink.rtnl import TC_H_ROOT
 from pyroute2.netlink.rtnl import rtprotos
@@ -101,8 +115,10 @@ from pyroute2.netlink.rtnl.rtmsg import rtmsg
 from pyroute2.netlink.rtnl.ndmsg import ndmsg
 from pyroute2.netlink.rtnl.brmsg import brmsg
 from pyroute2.netlink.rtnl.bomsg import bomsg
+from pyroute2.netlink.rtnl.dhcpmsg import dhcpmsg
 from pyroute2.netlink.rtnl.ifinfmsg import ifinfmsg
 from pyroute2.netlink.rtnl.ifaddrmsg import ifaddrmsg
+from pyroute2.netlink.rtnl.tuntapmsg import tuntapmsg
 from pyroute2.netlink.rtnl import IPRSocket
 
 from pyroute2.common import basestring
@@ -117,12 +133,15 @@ def transform_handle(handle):
     return handle
 
 
-class IPRoute(IPRSocket):
+class IPRouteMixin(object):
     '''
-    You can think of this class in some way as of plain old iproute2
-    utility.
+    `IPRouteMixin` should not be instantiated by itself. It is intended
+    to be used as a mixin class that provides iproute2-like API. You
+    should use `IPRoute` or `NetNS` classes.
 
-    It is an old-style library, that provides access to rtnetlink as is.
+    All following info you can consider as IPRoute info as well.
+
+    It is an old-school API, that provides access to rtnetlink as is.
     It helps you to retrieve and change almost all the data, available
     through rtnetlink::
 
@@ -223,6 +242,14 @@ class IPRoute(IPRSocket):
         msg['family'] = family
         return self.nlm_request(msg, RTM_GETADDR)
 
+    def get_dhcp(self, name, address=None):
+        msg = dhcpmsg()
+        msg['family'] = AF_INET
+        msg['attrs'] = [['DHCP_IFNAME', name]]
+        if address is not None:
+            msg['attrs'].append(['DHCP_ADDRESS', address])
+        return self.nlm_request(msg, RTM_GETDHCP)
+
     def get_bond(self, index, name):
         msg = bomsg()
         msg['index'] = index
@@ -437,7 +464,12 @@ class IPRoute(IPRSocket):
         command = commands.get(command, command)
 
         msg_flags = NLM_F_REQUEST | NLM_F_ACK | NLM_F_CREATE | NLM_F_EXCL
-        msg = ifinfmsg()
+        # intercept tuntap messages
+        if command == RTM_NEWLINK and kwarg.get('kind') == 'tuntap':
+            msg = tuntapmsg()
+            command = RTM_NEWTUNTAP
+        else:
+            msg = ifinfmsg()
         # index is required
         msg['index'] = kwarg.get('index')
 
@@ -454,7 +486,7 @@ class IPRoute(IPRSocket):
         msg['change'] = mask
 
         for key in kwarg:
-            nla = ifinfmsg.name2nla(key)
+            nla = type(msg).name2nla(key)
             if kwarg[key] is not None:
                 msg['attrs'].append([nla, kwarg[key]])
 
@@ -482,7 +514,8 @@ class IPRoute(IPRSocket):
 
         return self.nlm_request(msg, msg_type=command, msg_flags=0)
 
-    def addr(self, command, index, address, mask=24, family=None, scope=0):
+    def addr(self, command, index, address, mask=24,
+             family=None, scope=0, **kwarg):
         '''
         Address operations
 
@@ -525,6 +558,10 @@ class IPRoute(IPRSocket):
                             ['IFA_ADDRESS', address]]
         elif family == AF_INET6:
             msg['attrs'] = [['IFA_ADDRESS', address]]
+        for key in kwarg:
+            nla = ifaddrmsg.name2nla(key)
+            if kwarg[key] is not None:
+                msg['attrs'].append([nla, kwarg[key]])
         terminate = lambda x: x['header']['type'] == NLMSG_ERROR
         return self.nlm_request(msg,
                                 msg_type=command,
@@ -823,8 +860,8 @@ class IPRoute(IPRSocket):
             32006: from 10.64.75.141 fwmark 0xa lookup 15
             ...
         '''
-        if table < 0 or table > 254:
-            raise 'unsupported table number'
+        if table < 1:
+            raise ValueError('unsupported table number')
 
         commands = {'add': RTM_NEWRULE,
                     'del': RTM_DELRULE,
@@ -834,7 +871,7 @@ class IPRoute(IPRSocket):
 
         msg_flags = NLM_F_REQUEST | NLM_F_ACK | NLM_F_CREATE | NLM_F_EXCL
         msg = rtmsg()
-        msg['table'] = table
+        msg['table'] = table if table <= 255 else 252
         msg['family'] = family
         msg['type'] = rtypes[rtype]
         msg['scope'] = rtscopes[rtscope]
@@ -863,3 +900,14 @@ class IPRoute(IPRSocket):
         return self.nlm_request(msg, msg_type=command,
                                 msg_flags=msg_flags)
     # 8<---------------------------------------------------------------
+
+
+class IPRoute(IPRouteMixin, IPRSocket):
+    '''
+    Production class that provides iproute API over normal Netlink
+    socket.
+
+    You can think of this class in some way as of plain old iproute2
+    utility.
+    '''
+    pass
